searchNodes=[{"doc":"Wasmex is a fast and secure WebAssembly and WASI runtime for Elixir. It enables lightweight WebAssembly containers to be run in your Elixir backend. This is the main module, providing most of the needed API to run WASM binaries. It uses wasmtime to execute WASM binaries through a Rust NIF. Each WASM module must be compiled from a .wasm or '.wat' file. A compiled WASM module can be instantiated which usually happens in a GenServer . To start the GenServer, start_link/1 is used: iex&gt; bytes = File . read! ( TestHelper . wasm_test_file_path ( ) ) iex&gt; { :ok , instance_pid } = Wasmex . start_link ( %{ bytes : bytes } ) iex&gt; Wasmex . call_function ( instance_pid , &quot;sum&quot; , [ 50 , - 8 ] ) { :ok , [ 42 ] } Memory of a WASM instance can be read/written using Wasmex.Memory : iex&gt; { :ok , pid } = Wasmex . start_link ( %{ bytes : File . read! ( TestHelper . wasm_test_file_path ( ) ) } ) iex&gt; { :ok , store } = Wasmex . store ( pid ) iex&gt; { :ok , memory } = Wasmex . memory ( pid ) iex&gt; index = 4 iex&gt; Wasmex.Memory . set_byte ( store , memory , index , 42 ) iex&gt; Wasmex.Memory . get_byte ( store , memory , index ) 42 See start_link/1 for starting and configuring a WASM instance and call_function/3 for details about calling WASM functions.","ref":"Wasmex.html","title":"Wasmex","type":"module"},{"doc":"Calls a function with the given name and params on the WASM instance and returns its results. Example iex&gt; wat = &quot;(module ...&gt; (func $helloWorld (result i32) (i32.const 42)) ...&gt; (export \\&quot; hello_world \\&quot; (func $helloWorld)) ...&gt; )&quot; iex&gt; { :ok , pid } = Wasmex . start_link ( %{ bytes : wat } ) iex&gt; Wasmex . call_function ( pid , &quot;hello_world&quot; , [ ] ) { :ok , [ 42 ] } String Handling Strings are common candidates for function parameters and return values. However, they can not be used directly when calling WASM functions, because WASM only knows number data types. Since Strings are just &quot;a bunch of bytes&quot;, we can write these bytes into memory and give our WASM function a pointer to that memory location. Strings as Function Parameters Given we have the following Rust function that returns the first byte of a string input compiled to WASM: #[no_mangle] pub extern &quot;C&quot; fn string_first_byte(bytes: *const u8, length: usize) -&gt; u8 { let slice = unsafe { slice::from_raw_parts(bytes, length) }; match slice.first() { Some(&amp;i) =&gt; i, None =&gt; 0, } } This WASM function can be called from Elixir: iex&gt; { :ok , pid } = Wasmex . start_link ( %{ bytes : File . read! ( TestHelper . wasm_test_file_path ( ) ) } ) iex&gt; { :ok , store } = Wasmex . store ( pid ) iex&gt; { :ok , memory } = Wasmex . memory ( pid ) iex&gt; index = 42 iex&gt; string = &quot;hello, world&quot; iex&gt; Wasmex.Memory . write_binary ( store , memory , index , string ) iex&gt; Wasmex . call_function ( pid , &quot;string_first_byte&quot; , [ index , String . length ( string ) ] ) { :ok , [ 104 ] } # 104 is the letter &quot;h&quot; in ASCII/UTF-8 encoding Please not that Elixir and Rust assume Strings to be valid UTF-8. Take care when handling other encodings. Strings as Function Return Values Given we have the following Rust function compiled to WASM (again, copied from our test code): #[no_mangle] pub extern &quot;C&quot; fn string() -&gt; *const u8 { b&quot;Hello, World!&quot;.as_ptr() } This function returns a pointer to its memory. This memory location contains the String &quot;Hello, World!&quot;. This is how we would receive this String in Elixir: iex&gt; { :ok , pid } = Wasmex . start_link ( %{ bytes : File . read! ( TestHelper . wasm_test_file_path ( ) ) } ) iex&gt; { :ok , store } = Wasmex . store ( pid ) iex&gt; { :ok , memory } = Wasmex . memory ( pid ) iex&gt; { :ok , [ pointer ] } = Wasmex . call_function ( pid , &quot;string&quot; , [ ] ) iex&gt; Wasmex.Memory . read_string ( store , memory , pointer , 13 ) &quot;Hello, World!&quot; Specifying a timeout The default timeout for call_function is 5 seconds, or 5000 milliseconds. When calling a long-running function, you can specify a timeout value (in milliseconds) for this call. iex&gt; wat = &quot;(module ...&gt; (func $helloWorld (result i32) (i32.const 42)) ...&gt; (export \\&quot; hello_world \\&quot; (func $helloWorld)) ...&gt; )&quot; iex&gt; { :ok , pid } = Wasmex . start_link ( %{ bytes : wat } ) iex&gt; Wasmex . call_function ( pid , &quot;hello_world&quot; , [ ] , 10_000 ) { :ok , [ 42 ] } In the example above, we specify a timeout of 10 seconds.","ref":"Wasmex.html#call_function/4","title":"Wasmex.call_function/4","type":"function"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Wasmex.html#child_spec/1","title":"Wasmex.child_spec/1","type":"function"},{"doc":"Returns whether a function export with the given name exists in the WASM instance. Examples iex&gt; wat = &quot;(module ...&gt; (func $helloWorld (result i32) (i32.const 42)) ...&gt; (export \\&quot; hello_world \\&quot; (func $helloWorld)) ...&gt; )&quot; iex&gt; { :ok , pid } = Wasmex . start_link ( %{ bytes : wat } ) iex&gt; Wasmex . function_exists ( pid , &quot;hello_world&quot; ) true iex&gt; Wasmex . function_exists ( pid , &quot;something_else&quot; ) false","ref":"Wasmex.html#function_exists/2","title":"Wasmex.function_exists/2","type":"function"},{"doc":"Returns the exported Wasmex.Memory of the given WASM instance. Example iex&gt; { :ok , pid } = Wasmex . start_link ( %{ bytes : File . read! ( TestHelper . wasm_test_file_path ( ) ) } ) iex&gt; { :ok , % Wasmex.Memory { } } = Wasmex . memory ( pid )","ref":"Wasmex.html#memory/1","title":"Wasmex.memory/1","type":"function"},{"doc":"Starts a GenServer which compiles and instantiates a WASM module from the given .wasm or .wat bytes. iex&gt; bytes = File . read! ( TestHelper . wasm_test_file_path ( ) ) iex&gt; { :ok , _pid } = Wasmex . start_link ( %{ bytes : bytes } ) Alternatively, a precompiled Wasmex.Module can be passed with its Wasmex.Store : iex&gt; { :ok , store } = Wasmex.Store . new ( ) iex&gt; { :ok , module } = Wasmex.Module . compile ( store , &quot;(module)&quot; ) iex&gt; { :ok , _pid } = Wasmex . start_link ( %{ store : store , module : module } ) Imports WASM imports may be given as an additional option. Imports are a map of namespace-name to namespaces. Each namespace is in turn a map of import-name to import. iex&gt; wat = &quot;(module ...&gt; (import \\&quot; IO \\&quot; \\&quot; inspect \\&quot; (func $log (param i32))) ...&gt; )&quot; iex&gt; io_inspect = fn ( %{ _memory : % Wasmex.Memory { } , _caller : % Wasmex.StoreOrCaller { } } = _context , i ) -&gt; ...&gt; IO . inspect ( i ) ...&gt; end iex&gt; imports = %{ ...&gt; IO : %{ ...&gt; inspect : { :fn , [ :i32 ] , [ ] , io_inspect } , ...&gt; } ...&gt; } iex&gt; { :ok , _pid } = Wasmex . start_link ( %{ bytes : wat , imports : imports } ) In the example above, we import the &quot;IO&quot; namespace. That namespace is a map of imports, in this case the inspect function, which is represented with a tuple of: the import type: :fn (a function), the functions parameter types: [:i32] , the functions return types: [] , and the function to be executed: fn (_context, i) -&gt; IO.inspect(i) end The first param the function receives is always the call context: %{ memory : % Wasmex.Memory { } , caller : % Wasmex.StoreOrCaller { } } = context The caller MUST be used instead of a store in Wasmex API functions. Wasmex might deadlock if the store is used instead of the caller (because running the WASM instance holds a Mutex lock on the store so we cannot use that store again during the execution of an imported function). The caller, however, MUST NOT be used outside of the imported functions scope. All other params are regular parameters as specified by the parameter type list. Valid parameter/return types are: :i32 a 32 bit integer :i64 a 64 bit integer :f32 a 32 bit float :f64 a 64 bit float WASI Optionally, modules can be run with WebAssembly System Interface (WASI) support. WASI functions are provided as native implementations by default but could be overridden with Elixir provided functions. iex&gt; { :ok , _pid } = Wasmex . start_link ( %{ bytes : &quot;(module)&quot; , wasi : true } ) It is possible to overwrite the default WASI functions using the imports map: iex&gt; imports = %{ ...&gt; wasi_snapshot_preview1 : %{ ...&gt; random_get : { :fn , [ :i32 , :i32 ] , [ :i32 ] , ...&gt; fn %{ memory : memory , caller : caller } , address , size -&gt; ...&gt; Enum . each ( 0 .. size , fn index -&gt; ...&gt; Wasmex.Memory . set_byte ( caller , memory , address + index , 0 ) ...&gt; end ) ...&gt; # We chose `4` as the random number with a fair dice roll ...&gt; Wasmex.Memory . set_byte ( caller , memory , address , 4 ) ...&gt; 0 ...&gt; end ...&gt; } ...&gt; } ...&gt; } iex&gt; { :ok , _pid } = Wasmex . start_link ( %{ bytes : &quot;(module)&quot; , imports : imports } ) In the example above, we overwrite the random_get function which is (as all other WASI functions) implemented in Rust. This way our Elixir implementation of random_get is used instead of the default WASI implementation. Oftentimes, WASI programs need additional inputs like environment variables, arguments, or file system access. These can configured by additional Wasmex.Wasi.WasiOptions : iex&gt; wasi_options = % Wasmex.Wasi.WasiOptions { ...&gt; args : [ &quot;hello&quot; , &quot;from elixir&quot; ] , ...&gt; env : %{ ...&gt; &quot;A_NAME_MAPS&quot; =&gt; &quot;to a value&quot; , ...&gt; &quot;THE_TEST_WASI_FILE&quot; =&gt; &quot;prints all environment variables&quot; ...&gt; } , ...&gt; preopen : [ % Wasmex.Wasi.PreopenOptions { path : &quot;lib&quot; , alias : &quot;src&quot; } ] ...&gt; } iex&gt; { :ok , _pid } = Wasmex . start_link ( %{ bytes : &quot;(module)&quot; , wasi : wasi_options } ) It is also possible to capture stdout, stdin, or stderr of a WASI program using pipes: iex&gt; { :ok , stdin } = Wasmex.Pipe . new ( ) iex&gt; { :ok , stdout } = Wasmex.Pipe . new ( ) iex&gt; wasi_options = % Wasmex.Wasi.WasiOptions { ...&gt; args : [ &quot;wasmex&quot; , &quot;echo&quot; ] , ...&gt; stdin : stdin , ...&gt; stdout : stdout ...&gt; } iex&gt; { :ok , pid } = Wasmex . start_link ( %{ bytes : File . read! ( TestHelper . wasi_test_file_path ( ) ) , wasi : wasi_options } ) iex&gt; Wasmex.Pipe . write ( stdin , &quot;Hey! It compiles! Ship it!&quot; ) iex&gt; Wasmex.Pipe . seek ( stdin , 0 ) iex&gt; { :ok , _ } = Wasmex . call_function ( pid , :_start , [ ] ) iex&gt; Wasmex.Pipe . seek ( stdout , 0 ) iex&gt; Wasmex.Pipe . read ( stdout ) &quot;Hey! It compiles! Ship it! \\n &quot; In the example above, we call a WASI program which echoes a line from stdin back to stdout.","ref":"Wasmex.html#start_link/1","title":"Wasmex.start_link/1","type":"function"},{"doc":"Returns the Wasmex.Store of the WASM instance. Example iex&gt; { :ok , pid } = Wasmex . start_link ( %{ bytes : File . read! ( TestHelper . wasm_test_file_path ( ) ) } ) iex&gt; { :ok , % Wasmex.StoreOrCaller { } } = Wasmex . store ( pid )","ref":"Wasmex.html#store/1","title":"Wasmex.store/1","type":"function"},{"doc":"Instantiates a WASM module and allows calling exported functions on it. In the majority of cases, you will not need to use this module directly but use the main module Wasmex instead. This module expects to be executed within GenServer context which Wasmex sets up.","ref":"Wasmex.Instance.html","title":"Wasmex.Instance","type":"module"},{"doc":"Calls a function the given name exported by the WASM instance with the given params . The WASM function will be invoked asynchronously in a new OS thread. The calling Process/GenServer will receive a {:returned_function_call, result, from} message once execution finishes. The result either is an {:error, reason} or :ok . call_exported_function/5 assumes to be called within a GenServer context, it expects a from argument as given by GenServer.handle_call/3 . from is returned unchanged to allow the wrapping GenServer to reply to their caller. A BadArg exception may be thrown when given unexpected input data. Function parameters Parameters for WASM functions are automatically casted to WASM values. Note that WebAssembly only knows number datatypes (floats and integers of various sizes). You can pass arbitrary data to WebAssembly by writing that data into an instances Wasmex.Memory . The memory/2 function returns the instances memory. Example iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; Wasmex.Instance . call_exported_function ( store , instance , &quot;sum&quot; , [ 1 , 2 ] , :from ) :ok iex&gt; receive do ...&gt; { :returned_function_call , { :ok , [ 3 ] } , :from } -&gt; :ok ...&gt; after ...&gt; 1000 -&gt; raise &quot;message_expected&quot; ...&gt; end","ref":"Wasmex.Instance.html#call_exported_function/5","title":"Wasmex.Instance.call_exported_function/5","type":"function"},{"doc":"Whether the WASM instance exports a function with the given name . Example iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; Wasmex.Instance . function_export_exists ( store , instance , &quot;sum&quot; ) true iex&gt; Wasmex.Instance . function_export_exists ( store , instance , &quot;does_not_exist&quot; ) false","ref":"Wasmex.Instance.html#function_export_exists/3","title":"Wasmex.Instance.function_export_exists/3","type":"function"},{"doc":"Returns the Wasmex.Memory of the WASM instance . Example iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , % Wasmex.Memory { } } = Wasmex.Instance . memory ( store , instance )","ref":"Wasmex.Instance.html#memory/2","title":"Wasmex.Instance.memory/2","type":"function"},{"doc":"Instantiates a WASM module with the given imports. Returns the instantiated WASM instance. The import parameter is a nested map of WASM namespaces. Each namespace consists of a name and a map of function names to function signatures. Function signatures are a tuple of the form {:fn, arg_types, return_types, callback} . Where arg_types and return_types are lists of :i32 , :i64 , :f32 , :f64 . Each callback function receives a context map as the first argument followed by the arguments specified in its signature. context has the following keys: :memory - The default exported Wasmex.Memory of the WASM instance :caller - The caller of the WASM instance which MUST be used instead of a Wasmex.Store in all Wasmex functions called from within the callback. Failure to do so will result in a deadlock. The caller MUST NOT be used outside of the callback. Example This example instantiates a WASM module with one namespace env having three imported functions imported_sum3 , imported_sumf , and imported_void . The imported function imported_sum3 takes three :i32 (32 bit integer) arguments and returns a :i32 number. Its implementation is defined by the callback function fn _context, a, b, c -&gt; a + b + c end . iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; imports = %{ ...&gt; &quot;env&quot; =&gt; ...&gt; %{ ...&gt; &quot;imported_sum3&quot; =&gt; { :fn , [ :i32 , :i32 , :i32 ] , [ :i32 ] , fn _context , a , b , c -&gt; a + b + c end } , ...&gt; &quot;imported_sumf&quot; =&gt; { :fn , [ :f32 , :f32 ] , [ :f32 ] , fn _context , a , b -&gt; a + b end } , ...&gt; &quot;imported_void&quot; =&gt; { :fn , [ ] , [ ] , fn _context -&gt; nil end } ...&gt; } ...&gt; } iex&gt; { :ok , % Wasmex.Instance { } } = Wasmex.Instance . new ( store , module , imports )","ref":"Wasmex.Instance.html#new/3","title":"Wasmex.Instance.new/3","type":"function"},{"doc":"","ref":"Wasmex.Instance.html#t:t/0","title":"Wasmex.Instance.t/0","type":"type"},{"doc":"Memory is a linear array of bytes to store WASM values. The Memory module provides functions to read and write to this array. Memory is accessible through Wasmex.Instance.memory/2 , Wasmex.Memory.from_instance/2 , or as the caller context of an imported function (see Wasmex.Instance.call_exported_function/5 ). iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , memory } = Wasmex.Instance . memory ( store , instance ) iex&gt; Wasmex.Memory . set_byte ( store , memory , 0 , 42 ) iex&gt; Wasmex.Memory . get_byte ( store , memory , 0 ) 42 WASM memory is organized in pages of 64kb and may be grown by additional pages.","ref":"Wasmex.Memory.html","title":"Wasmex.Memory","type":"module"},{"doc":"Returns the exported memory resource of the given Wasmex.Instance . Example iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , % Wasmex.Memory { } } = Wasmex.Memory . from_instance ( store , instance )","ref":"Wasmex.Memory.html#from_instance/2","title":"Wasmex.Memory.from_instance/2","type":"function"},{"doc":"Returns the byte at the given index . Example Set a value at memory position 0 and read it back: iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , memory } = Wasmex.Memory . from_instance ( store , instance ) iex&gt; Wasmex.Memory . set_byte ( store , memory , 0 , 42 ) iex&gt; Wasmex.Memory . get_byte ( store , memory , 0 ) 42","ref":"Wasmex.Memory.html#get_byte/3","title":"Wasmex.Memory.get_byte/3","type":"function"},{"doc":"Grows the amount of available memory by the given number of pages. Returns the number of previously available pages. A page has a size of 64 kB or 65,536 bytes. Returns an error if memory could not be grown. Example iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , memory } = Wasmex.Memory . from_instance ( store , instance ) iex&gt; Wasmex.Memory . grow ( store , memory , 1 ) 17","ref":"Wasmex.Memory.html#grow/3","title":"Wasmex.Memory.grow/3","type":"function"},{"doc":"Reads the given number of bytes from the given memory at the given index. Returns the read bytes as a binary. Example Reads 5 bytes from memory position 0 , given it contains the 5 ASCII characters forming &quot;hello&quot;. iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , memory } = Wasmex.Memory . from_instance ( store , instance ) iex&gt; Wasmex.Memory . write_binary ( store , memory , 0 , &quot;hello&quot; ) iex&gt; Wasmex.Memory . read_binary ( store , memory , 0 , 5 ) &quot;hello&quot; iex&gt; Wasmex.Memory . read_binary ( store , memory , 3 , 2 ) &quot;lo&quot;","ref":"Wasmex.Memory.html#read_binary/4","title":"Wasmex.Memory.read_binary/4","type":"function"},{"doc":"Reads the given number of bytes from the given memory at the given index. Returns the read bytes as a string. Example Reads 5 bytes from memory position 0 , given it contains the 5 ASCII characters forming &quot;hello&quot;. iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , memory } = Wasmex.Memory . from_instance ( store , instance ) iex&gt; Wasmex.Memory . write_binary ( store , memory , 0 , &quot;hello&quot; ) iex&gt; Wasmex.Memory . read_string ( store , memory , 0 , 5 ) &quot;hello&quot; iex&gt; Wasmex.Memory . read_string ( store , memory , 3 , 2 ) &quot;lo&quot;","ref":"Wasmex.Memory.html#read_string/4","title":"Wasmex.Memory.read_string/4","type":"function"},{"doc":"Sets the byte at the given index to the given value . Example Set a value at memory position 0 and read it back: iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , memory } = Wasmex.Memory . from_instance ( store , instance ) iex&gt; Wasmex.Memory . set_byte ( store , memory , 0 , 42 ) iex&gt; Wasmex.Memory . get_byte ( store , memory , 0 ) 42","ref":"Wasmex.Memory.html#set_byte/4","title":"Wasmex.Memory.set_byte/4","type":"function"},{"doc":"Returns the size in bytes of the given memory. Note that the size of the memory is always a multiple of 64kb (one page). Example iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , memory } = Wasmex.Memory . from_instance ( store , instance ) iex&gt; Wasmex.Memory . size ( store , memory ) 1114112 # in bytes (17 pages of 64 kB)","ref":"Wasmex.Memory.html#size/2","title":"Wasmex.Memory.size/2","type":"function"},{"doc":"Writes the given binary into the memory at the given index . Example Writes 5 bytes representing the ASCII characters for &quot;hello&quot; at memory position 0 . iex&gt; %{ store : store , module : module } = TestHelper . wasm_module ( ) iex&gt; { :ok , instance } = Wasmex.Instance . new ( store , module , %{ } ) iex&gt; { :ok , memory } = Wasmex.Memory . from_instance ( store , instance ) iex&gt; Wasmex.Memory . write_binary ( store , memory , 0 , &quot;hello&quot; ) :ok","ref":"Wasmex.Memory.html#write_binary/4","title":"Wasmex.Memory.write_binary/4","type":"function"},{"doc":"","ref":"Wasmex.Memory.html#t:t/0","title":"Wasmex.Memory.t/0","type":"type"},{"doc":"A compiled WebAssembly module. A WASM Module contains stateless WebAssembly code that has already been compiled and can be instantiated multiple times.","ref":"Wasmex.Module.html","title":"Wasmex.Module","type":"module"},{"doc":"Compiles a WASM module from it's WASM (a .wasm file) or WAT (a .wat file) representation. Compiled modules can be instantiated using Wasmex.start_link/1 or Instance.new/3 . Since module compilation takes time and resources but instantiation is comparatively cheap, it may be a good idea to compile a module once and instantiate it often if you want to run a WASM binary multiple times. Example Read a WASM file and compile it into a WASM module. Use the compiled module to start a running Wasmex.Instance . iex&gt; { :ok , store } = Wasmex.Store . new ( ) iex&gt; { :ok , module } = Wasmex.Module . compile ( store , File . read! ( TestHelper . wasm_test_file_path ( ) ) ) iex&gt; { :ok , _pid } = Wasmex . start_link ( %{ store : store , module : module } ) Modules can be compiled from WAT (WebAssembly Text) format: iex&gt; wat = &quot;(module)&quot; # minimal and not very useful iex&gt; { :ok , store } = Wasmex.Store . new ( ) iex&gt; { :ok , % Wasmex.Module { } } = Wasmex.Module . compile ( store , wat )","ref":"Wasmex.Module.html#compile/2","title":"Wasmex.Module.compile/2","type":"function"},{"doc":"Lists all exports of a WASM module. Returns a map which has the exports name (string) as key and export info-tuples as values. Info tuples always start with an atom indicating the exports type: :fn (function) :global :table :memory Further parts of the info tuple vary depending on the type. Example List the exported function &quot;hello_world()&quot; of a WASM module: iex&gt; { :ok , store } = Wasmex.Store . new ( ) iex&gt; wat = &quot;(module ...&gt; (func $helloWorld (result i32) (i32.const 42)) ...&gt; (export \\&quot; hello_world \\&quot; (func $helloWorld)) ...&gt; )&quot; iex&gt; { :ok , module } = Wasmex.Module . compile ( store , wat ) iex&gt; Wasmex.Module . exports ( module ) %{ &quot;hello_world&quot; =&gt; { :fn , [ ] , [ :i32 ] } , }","ref":"Wasmex.Module.html#exports/1","title":"Wasmex.Module.exports/1","type":"function"},{"doc":"Lists all imports of a WebAssembly module grouped by their module namespace. Returns a map of namespace names to namespaces with each namespace being a map again. A namespace is a map of imports with the import name as key and and info-tuple as value. Info tuples always start with an atom indicating the imports type: :fn (function) :global :table :memory Further parts of the info tuple vary depending on the type. Example Show that the WASM module imports a function &quot;inspect&quot; from the &quot;IO&quot; namespace: iex&gt; { :ok , store } = Wasmex.Store . new ( ) iex&gt; wat = &quot;(module ...&gt; (import \\&quot; IO \\&quot; \\&quot; inspect \\&quot; (func $log (param i32))) ...&gt; )&quot; iex&gt; { :ok , module } = Wasmex.Module . compile ( store , wat ) iex&gt; Wasmex.Module . imports ( module ) %{ &quot;IO&quot; =&gt; %{ &quot;inspect&quot; =&gt; { :fn , [ :i32 ] , [ ] } , } }","ref":"Wasmex.Module.html#imports/1","title":"Wasmex.Module.imports/1","type":"function"},{"doc":"Returns the name of the current module if a name is given. This name is normally set in the WASM bytecode by some compilers. Example iex&gt; { :ok , store } = Wasmex.Store . new ( ) iex&gt; wat = &quot;(module $hiFromTheDocs)&quot; # minimal and not very useful WASM module iex&gt; { :ok , module } = Wasmex.Module . compile ( store , wat ) iex&gt; Wasmex.Module . name ( module ) &quot;hiFromTheDocs&quot;","ref":"Wasmex.Module.html#name/1","title":"Wasmex.Module.name/1","type":"function"},{"doc":"Serializes a compiled WASM module into a binary. The generated binary can be deserialized back into a module using unsafe_deserialize/1 . It is unsafe do alter the binary in any way. See unsafe_deserialize/1 for safety considerations. Example Serializes a compiled module: iex&gt; { :ok , store } = Wasmex.Store . new ( ) iex&gt; { :ok , module } = Wasmex.Module . compile ( store , File . read! ( TestHelper . wasm_test_file_path ( ) ) ) iex&gt; { :ok , serialized } = Wasmex.Module . serialize ( module ) iex&gt; is_binary ( serialized ) true","ref":"Wasmex.Module.html#serialize/1","title":"Wasmex.Module.serialize/1","type":"function"},{"doc":"Deserializes a module from its binary representation. This function is inherently unsafe as the provided binary is going to be deserialized directly into Rust objects contains the WASM function assembly bodies and, if intercepted, a malicious actor could inject code into executable memory. Only pass binaries directly coming from serialize/1 , never any user input. Best case is it crashing the NIF, worst case is malicious input doing... malicious things. The deserialization must be done on the same CPU architecture as the serialization (e.g. don't serialize a x86_64-compiled module and deserialize it on ARM64). Example Serializes a compiled module and deserializes it again: iex&gt; { :ok , store } = Wasmex.Store . new ( ) iex&gt; { :ok , module } = Wasmex.Module . compile ( store , File . read! ( TestHelper . wasm_test_file_path ( ) ) ) iex&gt; { :ok , serialized } = Wasmex.Module . serialize ( module ) iex&gt; { :ok , % Wasmex.Module { } } = Wasmex.Module . unsafe_deserialize ( serialized )","ref":"Wasmex.Module.html#unsafe_deserialize/1","title":"Wasmex.Module.unsafe_deserialize/1","type":"function"},{"doc":"","ref":"Wasmex.Module.html#t:t/0","title":"Wasmex.Module.t/0","type":"type"},{"doc":"A Pipe is a memory buffer that can be used in exchange for a WASM file. Pipes have a read and write position which can be set using seek/2 . Example Pipes can be written to and read from: iex&gt; { :ok , pipe } = Wasmex.Pipe . new ( ) iex&gt; Wasmex.Pipe . write ( pipe , &quot;hello&quot; ) { :ok , 5 } iex&gt; Wasmex.Pipe . seek ( pipe , 0 ) iex&gt; Wasmex.Pipe . read ( pipe ) &quot;hello&quot; They can be used to capture stdout/stdin/stderr of WASI programs: iex&gt; { :ok , stdin } = Wasmex.Pipe . new ( ) iex&gt; { :ok , stdout } = Wasmex.Pipe . new ( ) iex&gt; { :ok , stderr } = Wasmex.Pipe . new ( ) iex&gt; Wasmex.Store . new_wasi ( % Wasmex.Wasi.WasiOptions { ...&gt; stdin : stdin , ...&gt; stdout : stdout , ...&gt; stderr : stderr , ...&gt; } )","ref":"Wasmex.Pipe.html","title":"Wasmex.Pipe","type":"module"},{"doc":"Creates and returns a new Pipe. Example iex&gt; { :ok , % Pipe { } } = Wasmex.Pipe . new ( )","ref":"Wasmex.Pipe.html#new/0","title":"Wasmex.Pipe.new/0","type":"function"},{"doc":"Reads all available bytes from the Pipe and returns them as a binary. This function does not block if there are no bytes available. Reading starts at the current read position, see seek/2 , and forwards the read position to the end of the Pipe. The read bytes are not erased and can be read again after seeking back. Example iex&gt; { :ok , pipe } = Wasmex.Pipe . new ( ) iex&gt; Wasmex.Pipe . write ( pipe , &quot;hello&quot; ) iex&gt; Wasmex.Pipe . read ( pipe ) # current position is at EOL, nothing more to read &quot;&quot; iex&gt; Wasmex.Pipe . seek ( pipe , 0 ) iex&gt; Wasmex.Pipe . read ( pipe ) &quot;hello&quot; iex&gt; Wasmex.Pipe . seek ( pipe , 3 ) iex&gt; Wasmex.Pipe . read ( pipe ) &quot;lo&quot; iex&gt; Wasmex.Pipe . read ( pipe ) &quot;&quot;","ref":"Wasmex.Pipe.html#read/1","title":"Wasmex.Pipe.read/1","type":"function"},{"doc":"Sets the read/write position of the Pipe to the given position. The position is given as a number of bytes from the start of the Pipe. Example iex&gt; { :ok , pipe } = Wasmex.Pipe . new ( ) iex&gt; Wasmex.Pipe . write ( pipe , &quot;hello&quot; ) iex&gt; Wasmex.Pipe . seek ( pipe , 0 ) :ok iex&gt; Wasmex.Pipe . read ( pipe ) &quot;hello&quot;","ref":"Wasmex.Pipe.html#seek/2","title":"Wasmex.Pipe.seek/2","type":"function"},{"doc":"Returns the current size of the Pipe in bytes. Example iex&gt; { :ok , pipe } = Wasmex.Pipe . new ( ) iex&gt; Wasmex.Pipe . size ( pipe ) 0 iex&gt; Wasmex.Pipe . write ( pipe , &quot;hello&quot; ) iex&gt; Wasmex.Pipe . size ( pipe ) 5","ref":"Wasmex.Pipe.html#size/1","title":"Wasmex.Pipe.size/1","type":"function"},{"doc":"Writes the given binary into the pipe. Writing starts at the current write position, see seek/2 , and forwards it. Example iex&gt; { :ok , pipe } = Wasmex.Pipe . new ( ) iex&gt; Wasmex.Pipe . write ( pipe , &quot;hello&quot; ) { :ok , 5 } iex&gt; Wasmex.Pipe . seek ( pipe , 0 ) iex&gt; Wasmex.Pipe . read ( pipe ) &quot;hello&quot;","ref":"Wasmex.Pipe.html#write/2","title":"Wasmex.Pipe.write/2","type":"function"},{"doc":"","ref":"Wasmex.Pipe.html#t:t/0","title":"Wasmex.Pipe.t/0","type":"type"},{"doc":"A Store is a collection of WASM instances and host-defined state. All WASM instances and items will be attached to and refer to a Store. For example instances, functions, globals, and tables are all attached to a Store. Instances are created by instantiating a Module within a Store. Many functions of the Wasmex API require a Store in the form of a Wasmex.StoreOrCaller to be passed in. A Store is intended to be a short-lived object in a program. No form of GC is implemented at this time so once an instance is created within a Store it will not be deallocated until the Store itself is garbage collected. This makes Store unsuitable for creating an unbounded number of instances in it because Store will never release this memory. It’s recommended to have a Store correspond roughly to the lifetime of a “main instance”.","ref":"Wasmex.Store.html","title":"Wasmex.Store","type":"module"},{"doc":"Creates a new WASM store. Returns a Wasmex.StoreOrCaller even though we know it’s definitely a Store. This allows Elixir-provided imported functions, which only have a &quot;Caller&quot;, to use the same Wasmex APIs. Examples iex&gt; { :ok , % Wasmex.StoreOrCaller { } } = Wasmex.Store . new ( )","ref":"Wasmex.Store.html#new/0","title":"Wasmex.Store.new/0","type":"function"},{"doc":"Creates a new WASM store with WASI support. Returns a Wasmex.StoreOrCaller even though we know it’s definitely a Store. This allows Elixir-provided imported functions, which only have a &quot;Caller&quot;, to use the same Wasmex APIs. See Wasmex.Wasi.WasiOptions for WASI configuration options. Examples iex&gt; { :ok , % Wasmex.StoreOrCaller { } } = Wasmex.Store . new_wasi ( % Wasmex.Wasi.WasiOptions { } )","ref":"Wasmex.Store.html#new_wasi/1","title":"Wasmex.Store.new_wasi/1","type":"function"},{"doc":"Either a Wasmex.Store or &quot;Caller&quot; for imported functions. A Store is a collection of WASM instances and host-defined state, see Wasmex.Store . A Caller takes the place of a Store in imported function calls. If a Store is needed in Elixir-provided imported functions, always use the provided Caller because using the Store will cause a deadlock (the running WASM instance locks the Stores Mutex).","ref":"Wasmex.StoreOrCaller.html","title":"Wasmex.StoreOrCaller","type":"module"},{"doc":"","ref":"Wasmex.StoreOrCaller.html#t:t/0","title":"Wasmex.StoreOrCaller.t/0","type":"type"},{"doc":"Options for preopening a directory. Likely to be extended with read/write permissions once wasmtime supports them. Options :path - The path to the directory to preopen :alias - The alias to use for the directory. The directory will be available at this path in the WASI filesystem. If not specified, the directory will be available at its real path. Example iex&gt; Wasmex.Store . new_wasi ( % Wasmex.Wasi.WasiOptions { ...&gt; preopen : [ ...&gt; % PreopenOptions { path : &quot;/tmp&quot; , alias : &quot;temp&quot; } ...&gt; ] , ...&gt; } )","ref":"Wasmex.Wasi.PreopenOptions.html","title":"Wasmex.Wasi.PreopenOptions","type":"module"},{"doc":"","ref":"Wasmex.Wasi.PreopenOptions.html#t:t/0","title":"Wasmex.Wasi.PreopenOptions.t/0","type":"type"},{"doc":"Configures WASI support for a Wasmex.Store. Options :args - A list of command line arguments :env - A map of environment variables :preopen - A list of Wasmex.Wasi.PreopenOptions to preopen directories :stdin - A Wasmex.Pipe to use as stdin :stdout - A Wasmex.Pipe to use as stdout :stderr - A Wasmex.Pipe to use as stderr Example iex&gt; { :ok , stdin } = Wasmex.Pipe . new ( ) iex&gt; Wasmex.Store . new_wasi ( % WasiOptions { ...&gt; args : [ &quot;first param&quot; , &quot;second param&quot; ] , ...&gt; env : %{ &quot;env_key&quot; =&gt; &quot;env_value&quot; } , ...&gt; preopen : [ ...&gt; % Wasmex.Wasi.PreopenOptions { path : &quot;/tmp&quot; } ...&gt; ] , ...&gt; stdin : stdin , ...&gt; } )","ref":"Wasmex.Wasi.WasiOptions.html","title":"Wasmex.Wasi.WasiOptions","type":"module"},{"doc":"","ref":"Wasmex.Wasi.WasiOptions.html#t:t/0","title":"Wasmex.Wasi.WasiOptions.t/0","type":"type"}]